function P = parameterDef(audio, video, sizeVideoAtom, atomNum, varargin)
% QINGJU LIU, University of Surrey
% This parameter definition is for audio-visual dictionary learning.
% input:
%   audio: the synchronized audio power spectrum eg:513 x 10000.
%   video: the video stream eg: 50 x 50 x 300.
%   sizeVideoAtom: size of each phi_k^v
%   atomNum: maximum training dictionary size (K)
%   Varargin:
%       atomMinNum:     converged minimum dicationary size
%       iterMax:        maximum # iterations
%       thresholdBench: Change threshold at these iterations
%       thresholdArray: with values in thresholdArray
%       dispMode:    text display
%       figureMode:     figure display
%       figureIter:     ditto
%       SimilarityArray:Which types of objective function to be used
%       sizeVideoAtom:
%       sparsity:       to calculate the # atoms located
%       iterStop:       when begin to consider stopping the iteration
%       stationaryMax:  stop the iteration with stationaryMax repeated codings
%       overlapBench:   change the overlap constraints at thes iterations
%       noOverlapCell: with values in noOverlapCell
%       scanIndex:      just search the scanIndex area for the coding stage
%       N:              # atoms used to represent the AV-sequence
%       changeAtomBench:stop the atom change in this iteration


if numel(audio) == length(audio),   % 1 D
    audioFlag = 1;
    audio_length = length(audio);
else                                % 2 D
    audioFlag = 2;
    [halfNFFT, audio_length] = size(audio);
end
[Vr Vc Frame] = size(video);
FsaVsFsv = audio_length/Frame;      % AV sampling rates ratio

%% Following are optional input parameters

%%% the display mode and saving result mode parameters
%%% We choose whether or not to show and save the experimental resluts
[dispMode, figureMode, figureIter, saveMode, saveIter, seedNumber] ...
    = process_options(varargin, ...
    'dispMode', 1, 'figureMode', 0, 'figureIter', 5, 'saveMode', 0, 'saveIter', 5, 'seedNumber',[]);

%%% Changing Bench parameters definition
%%% when we coding the AV sequence using MP, we will consider the Matching
%%% coefficents as zeros when it is less than the threshold.
%%% Two atoms are not allowed to have overlap area bigger than noOverlap.
%%% the threshold and noOverlap will change at different iterations defined
%%% in the Bench.
[thresholdBench, thresholdCell, realThresholdCell, noOverlapBench, noOverlapCell] ...
    = process_options(varargin, ...
    'thresholdBench', [2 5 10], 'thresholdCell', {[0.001 0.001], [0.01 0.01], [0.1 0.1]}, ...
    'realThresholdCell', [], 'noOverlapBench', [], 'noOverlapCell',[]);

%%% Stationary parameters
%%% We will consider whether the system has converged to stationary when
%%% after the iterStop-th iteration, and with two successive
%%% stationary means convergence. The converged atom Number must be bigger
%%% than #atomMinNum; otherwise, one atom will be splited if iter>
%%% #changeAtomBench.
[atomMinNum, changeAtomBench, iterMax, iterStop, varThreshold]...
    = process_options(varargin, ...
    'atomMinNum', 1, 'changeAtomBench', [], 'iterMax', 1000, 'iterStop', 10, 'varThreshold', 0.5);

%%% Other general parameter definition
[similarityArray,sizeAudioAtom, N, sparsity, scanIndex, hopSize, frameStartEnd,thresholdNum] = ...
    process_options(varargin,...
    'similarityArray',[1 1], 'sizeAudioAtom', [], 'atomsUsedNum', [], 'sparsity', 0.8, ...
    'scanIndex', [], 'hopSize', [],...
    'frameStartEnd', [1 sizeVideoAtom(3)], 'thresholdNum', []);

atomMinNum = min(atomMinNum,atomNum);

%% Default input parameters that needs calculation
%%%***********************************************************************
% audio atom size is calculated.
if isempty(sizeAudioAtom),
    sizeAudioAtom = calAudioAtomSizeGivenVideoAtom(audio,video,sizeVideoAtom);
end
%%%***********************************************************************
% we use N kernels to approximate the audiovisual signal.
if isempty(N),
    frame = sizeVideoAtom(3);
    N = round(Frame/frame*sparsity); % where 0.5 is a sparsity measure:)
end
%%%***********************************************************************
threshold = [0.000001 0.000001];
realThreshold = [];
%%%***********************************************************************
% two atoms are not allowed to have any overlap.
noOverlap = sizeVideoAtom-1;
if isempty(noOverlapBench) || isempty(noOverlapCell),
    noOverlapBench = [];noOverlapCell={};
elseif noOverlapBench(1) == 1,
    noOverlap = noOverlapCell{1};
end
%%%***********************************************************************
% After this iteration, no atom change is allowed
if isempty(changeAtomBench)
    changeAtomBench = max(round(thresholdBench(end)*2),5);
end
%%%***********************************************************************
% find the patches where the video matching is applied.
if ~isempty(scanIndex),
else
    frameDynamic = frame;
    scanIndex = initialscanIndex(audio, video, sizeVideoAtom,frameDynamic,0);
end


%%%***********************************************************************
% initialization of Da and Dv
[Da Dv] = atomInit(sizeAudioAtom,sizeVideoAtom,atomNum,video,similarityArray,seedNumber);
ExistingFigN = length(findall(0,'Type','Figure'));

[dvr dvc frame Nk] = size(Dv);
% YXZsearch
ySearch = min(Vr-dvr+1,size(scanIndex,1));
xSearch = min(Vc-dvc+1,size(scanIndex,2));
zSearch = min(Frame-frame+1,size(scanIndex,3));
yxzSearch = [ySearch xSearch zSearch];

%%%***********************************************************************
%
if isempty(hopSize),
    yHop = min(ySearch, max(1,round(sizeVideoAtom(1)/10)));
    xHop = min(xSearch, max(1,round(sizeVideoAtom(2)/10)));
    zHop = min(zSearch, max(1,round(sizeVideoAtom(3)/8)));
    hopSize = [yHop,xHop,zHop];
end

if audioFlag==1,                    % 1D
    dar = size(Da,1);
else                            	% 2D
    dar = size(Da,2);
end
nint = round(dar/frame);

P.atomNum = atomNum;
P.sizeAudioAtom = sizeAudioAtom;
P.sizeVideoAtom = sizeVideoAtom;
P.FsaVsFsv = FsaVsFsv;
P.audioFlag = audioFlag;
if audioFlag==2,
    P.halfNFFT = halfNFFT;
else
    P.halfNFFT = [];
end

P.dispMode = dispMode;
P.figureMode = figureMode;
P.figureIter = figureIter;
P.saveMode = saveMode;
P.saveIter = saveIter;
P.ExistingFigN = ExistingFigN;

P.thresholdBench = thresholdBench;
P.thresholdCell = thresholdCell;  % if realThresholdCell exists,
% then there is no need to use thresholdCell.
% We can define realThresholdCell for normalized similarities,
% such as SSIM and correlation
P.realThresholdCell = realThresholdCell;
P.threshold = threshold;
P.realThreshold = realThreshold;
P.noOverlapBench = noOverlapBench;
P.noOverlapCell = noOverlapCell;
P.noOverlap = noOverlap;

P.atomMinNum = atomMinNum;
P.atomMinNumUse = atomMinNum;
P.changeAtomBench = changeAtomBench;
P.iterMax = iterMax;
P.iterStop = iterStop;
% P.stationaryMax = stationaryMax;
P.varThreshold = varThreshold;

P.similarityArray = similarityArray;
P.atomsUsedNum = N;
P.sparsity = sparsity;
P.scanIndex = scanIndex;
P.paramsOld = [];  % results of projecting each atom to the AV sequence.
P.projectionIndex = ones(2,atomNum);
P.similarityIndex = zeros(2,atomNum);
P.hopSize = hopSize;
P.frameStartEnd = frameStartEnd;
P.yxzSearch = yxzSearch;
P.frame = frame;
P.nint = nint;
P.dar = dar;
P.Da = Da;
P.Dv = Dv;
P.thresholdNum = thresholdNum;

if any(ismember([2], P.similarityArray(2))), %%% using SSIM for visual similarity calculation
    u1 = zeros(size(scanIndex));
    sigma1_sqr = zeros(size(scanIndex));
    frameStart = P.frameStartEnd(1);
    frameEnd = P.frameStartEnd(2);
    subVideoAddIndex = repmat((0:dvr-1)',1,dvc)+repmat((0:dvc-1)*Vr,dvr,1);
    ffframe = frameEnd-frameStart+1;
    subVideoAddIndex = repmat(subVideoAddIndex,[1 1 ffframe]) + ...
        repmat(reshape((frameStart-1:frameEnd-1)*Vr*Vc,[1 1 ffframe]),dvr,dvc);
    subVideoAddIndex = subVideoAddIndex(:); MM = length(subVideoAddIndex);
    for z = 1:zSearch, % z shift
        tempScanZ = scanIndex(:,:,z);
        if any(tempScanZ(:)),
            videoScan = false(Vr,Vc);
            videoScan(1:ySearch,1:xSearch) = tempScanZ;
            videoNonZeroIndex = find(videoScan); NN = length(videoNonZeroIndex);
            VideoAddIndex = repmat(subVideoAddIndex,1,NN) + ...
                repmat(videoNonZeroIndex(:)'+(z-1)*Vr*Vc,MM,1);
            videoTensor = video(VideoAddIndex);
            tempU1 = mean(videoTensor);
            temp = zeros(size(tempScanZ));
            temp(tempScanZ) = tempU1;
            u1(:,:,z) = temp;
            temp = zeros(size(tempScanZ));
            temp(tempScanZ) = mean(videoTensor.^2) - tempU1.^2;
            sigma1_sqr(:,:,z) = temp;
        end
    end
    
    Dv = reshape(Dv(:,:,frameStart:frameEnd,:),[],Nk);
    uDv = mean(Dv);
    sigmaDv_sqr = mean(Dv.^2)-uDv.^2;
    
    P.u1 = u1;
    P.sigma1_sqr = sigma1_sqr;
    P.uDv = uDv;
    P.sigmaDv_sqr = sigmaDv_sqr;
else
    P.u1 = [];
    P.sigma1_sqr = [];
    P.uDv = ones(1,Nk);          % which will not be used
    P.sigmaDv_sqr = ones(1,Nk);  % which will not be used
end



P.seedNumber = seedNumber; 
end


